locals {
  allowed_aws_services_for_sns_published = split(",", var.allowed_aws_services_for_sns_published)
}
resource "aws_sns_topic" "sns" {
  name              = var.topic_name
  kms_master_key_id = var.kms_master_key_id

  tags = var.tags
}

# TODO if we need custom policy and the below isn't enough then we can expose json policy in the chart
resource "aws_sns_topic_policy" "topic_policy" {
  arn    = aws_sns_topic.sns.arn
  policy = var.access_policy_statements ? file("access-policy.json") : data.aws_iam_policy_document.aws_sns_topic_policy.json
}

data "aws_iam_policy_document" "aws_sns_topic_policy" {
  policy_id = "SNSTopicsPub"
  statement {
    effect    = "Allow"
    actions   = ["sns:Publish"]
    resources = [aws_sns_topic.sns.arn]

    principals {
      type        = "Service"
      identifiers = local.allowed_aws_services_for_sns_published
    }
  }
}

resource "aws_iam_policy" "publisher-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-publisher"
  path        = "/"
  description = "SNS topic publish policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "sns:Publish"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_sns_topic.sns.arn}"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}

#TODO add support for DLQ if requested
//data "aws_iam_policy_document" "sqs-queue-policy" {
//  count = var.sqs_dlq_enabled ? 1 : 0
//
//  policy_id = "${join("", aws_sqs_queue.dead_letter_queue.*.arn)}/SNSDeadLetterQueue"
//
//  statement {
//    effect    = "Allow"
//    actions   = ["SQS:SendMessage"]
//    resources = [join("", aws_sqs_queue.dead_letter_queue.*.arn)]
//
//    principals {
//      type        = "AWS"
//      identifiers = ["*"]
//    }
//
//    condition {
//      test     = "ArnEquals"
//      variable = "aws:SourceArn"
//      values   = [aws_sns_topic.sns.arn]
//    }
//  }
//}

# TODO add support for for DLQ if requested
//resource "aws_sqs_queue" "dead_letter_queue" {
//  count = var.sqs_dlq_enabled ? 1 : 0
//
//  name                              = var.sqs_dlq_name
//  max_message_size                  = var.sqs_dlq_max_message_size
//  message_retention_seconds         = var.sqs_dlq_message_retention_seconds
//  kms_master_key_id                 = var.sqs_queue_kms_master_key_id
//  kms_data_key_reuse_period_seconds = var.sqs_queue_kms_data_key_reuse_period_seconds
//  tags                              = var.tags
//}